@@ -16,14 +16,22 @@ extern crate hyper;
 extern crate openstack;
 
 use openstack::auth::{AuthMethod, self};
+use openstack::session::AuthenticatedClient;
 use openstack::utils;
 
 
 fn main() {
-    let mut identity = auth::Identity::from_env()
+    let identity = auth::Identity::from_env()
         .expect("Failed to create an identity provider from the environment");
     let http_client = utils::http_client();
     let token_info = identity.get_token(&http_client)
         .expect("Failed to get a token");
     println!("Received token: {}", token_info.token);
+
+    let auth_client = AuthenticatedClient::new(http_client, token_info);
+    let identity_url = identity.get_endpoint("identity",
+                                             Some("public"),
+                                             None,
+                                             &auth_client).unwrap();
+    println!("Identity public URL is {}", identity_url);
 }
