@@ -23,58 +23,14 @@ use hyper::client::IntoUrl;
 use hyper::error::ParseError;
 use hyper::header::ContentType;
 use hyper::status::StatusCode;
-use rustc_serialize::json;
 
-use super::base::{AuthError, AuthMethod, AuthToken, SubjectTokenHeader};
+use super::super::ApiError;
+use super::super::identity::catalog;
+use super::super::identity::protocol;
+use super::super::session::AuthenticatedClient;
+use super::base::{AuthMethod, AuthToken, SubjectTokenHeader};
 
 
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct Domain {
-    name: String
-}
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct User {
-    name: String,
-    password: String,
-    domain: Domain
-}
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct PasswordAuth {
-    user: User
-}
-
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct PasswordIdentity {
-    methods: Vec<String>,
-    password: PasswordAuth
-}
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct Project {
-    name: String,
-    domain: Domain
-}
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct ProjectScope {
-    project: Project
-}
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct ScopedAuth {
-    identity: PasswordIdentity,
-    scope: ProjectScope
-}
-
-#[derive(Clone, RustcDecodable, RustcEncodable)]
-struct AuthRoot {
-    auth: ScopedAuth
-}
-
-const PASSWORD_METHOD: &'static str = "password";
 const MISSING_USER: &'static str = "User information required";
 const MISSING_SCOPE: &'static str = "Unscoped tokens are not supported now";
 const MISSING_ENV_VARS: &'static str =
@@ -85,8 +41,8 @@ const MISSING_ENV_VARS: &'static str =
 #[derive(Clone)]
 pub struct Identity {
     auth_url: Url,
-    password_auth: Option<PasswordAuth>,
-    project_scope: Option<ProjectScope>
+    password_identity: Option<protocol::PasswordIdentity>,
+    project_scope: Option<protocol::ProjectScope>
 }
 
 /// Authentication method using Identity API V3.
@@ -95,16 +51,21 @@ pub struct Identity {
 #[derive(Clone)]
 pub struct IdentityAuthMethod {
     auth_url: Url,
-    body: AuthRoot
+    body: protocol::ProjectScopedAuthRoot
 }
 
-impl Identity{
+impl Identity {
+    /// Get a reference to the auth URL.
+    pub fn get_auth_url(&self) -> &Url {
+        &self.auth_url
+    }
+
     /// Create a password authentication against the given Identity service.
     pub fn new<U>(auth_url: U) -> Result<Identity, ParseError> where U: IntoUrl  {
         let real_url = try!(auth_url.into_url());
         Ok(Identity {
             auth_url: real_url,
-            password_auth: None,
+            password_identity: None,
             project_scope: None
         })
     }
@@ -114,15 +75,9 @@ impl Identity{
                                  domain_name: S3) -> Identity
             where S1: Into<String>, S2: Into<String>, S3: Into<String> {
         Identity {
-            password_auth: Some(PasswordAuth {
-                user: User {
-                    name: user_name.into(),
-                    password: password.into(),
-                    domain: Domain {
-                        name: domain_name.into()
-                    }
-                }
-            }),
+            password_identity: Some(protocol::PasswordIdentity::new(user_name,
+                                                                    password,
+                                                                    domain_name)),
             .. self
         }
     }
@@ -131,55 +86,42 @@ impl Identity{
     pub fn with_project_scope<S1, S2>(self, project_name: S1, domain_name: S2)
             -> Identity where S1: Into<String>, S2: Into<String> {
         Identity {
-            project_scope: Some(ProjectScope {
-                project: Project {
-                    name: project_name.into(),
-                    domain: Domain {
-                        name: domain_name.into()
-                    }
-                }
-            }),
+            project_scope: Some(protocol::ProjectScope::new(project_name,
+                                                            domain_name)),
             .. self
         }
     }
 
     /// Create an authentication method based on provided information.
-    pub fn create(self) -> Result<IdentityAuthMethod, AuthError> {
+    pub fn create(self) -> Result<IdentityAuthMethod, ApiError> {
         /// TODO: support more authentication methods (at least a token)
-        let password_auth = match self.password_auth {
+        let password_identity = match self.password_identity {
             Some(p) => p,
             None =>
-                return Err(AuthError::InsufficientCredentials(MISSING_USER))
+                return Err(ApiError::InsufficientCredentials(MISSING_USER))
         };
 
         /// TODO: support unscoped tokens
         let project_scope = match self.project_scope {
             Some(p) => p,
             None =>
-                return Err(AuthError::InsufficientCredentials(MISSING_SCOPE))
+                return Err(ApiError::InsufficientCredentials(MISSING_SCOPE))
         };
 
         Ok(IdentityAuthMethod {
             auth_url: self.auth_url,
-            body: AuthRoot {
-                auth: ScopedAuth {
-                    identity: PasswordIdentity {
-                        methods: vec![String::from(PASSWORD_METHOD)],
-                        password: password_auth
-                    },
-                    scope: project_scope
-                }
-            }
+            body: protocol::ProjectScopedAuthRoot::new(password_identity,
+                                                       project_scope)
         })
     }
 
     /// Create an authentication method from environment variables.
-    pub fn from_env() -> Result<IdentityAuthMethod, AuthError> {
+    pub fn from_env() -> Result<IdentityAuthMethod, ApiError> {
         let auth_url = try!(_get_env("OS_AUTH_URL"));
         let id = match Identity::new(&auth_url) {
             Ok(x) => x,
             Err(e) =>
-                return Err(AuthError::ProtocolError(HttpClientError::Uri(e)))
+                return Err(ApiError::ProtocolError(HttpClientError::Uri(e)))
         };
 
         let user_name = try!(_get_env("OS_USERNAME"));
@@ -197,19 +139,26 @@ impl Identity{
     }
 }
 
-fn _get_env(name: &str) -> Result<String, AuthError> {
+fn _get_env(name: &str) -> Result<String, ApiError> {
     env::var(name).or(
-        Err(AuthError::InsufficientCredentials(MISSING_ENV_VARS)))
+        Err(ApiError::InsufficientCredentials(MISSING_ENV_VARS)))
+}
+
+impl IdentityAuthMethod {
+    /// Get a reference to the auth URL.
+    pub fn get_auth_url(&self) -> &Url {
+        &self.auth_url
+    }
 }
 
 impl AuthMethod for IdentityAuthMethod {
     /// Verify authentication and generate an auth token.
-    fn get_token(&mut self, client: &Client) -> Result<AuthToken, AuthError> {
+    fn get_token(&self, client: &Client) -> Result<AuthToken, ApiError> {
         // TODO: allow /v3 postfix built into auth_url?
         let url = format!("{}/v3/auth/tokens", self.auth_url.to_string());
         debug!("Requesting a token for user {} from {}",
                self.body.auth.identity.password.user.name, url);
-        let body = json::encode(&self.body).unwrap();
+        let body = self.body.to_string().unwrap();
         let json_type = ContentType(mime!(Application/Json));
 
         let mut resp = try!(client.post(&url).body(&body)
@@ -223,19 +172,19 @@ impl AuthMethod for IdentityAuthMethod {
                 let header: Option<&SubjectTokenHeader> = resp.headers.get();
                 match header {
                     Some(ref value) => value.0.clone(),
-                    None => return Err(AuthError::ProtocolError(
+                    None => return Err(ApiError::ProtocolError(
                             HttpClientError::Header))
                 }
             },
             StatusCode::Unauthorized => {
                 warn!("Invalid credentials for user {}",
                       self.body.auth.identity.password.user.name);
-                return Err(AuthError::Unauthorized);
+                return Err(ApiError::Unauthorized);
             },
             other => {
                 error!("Unexpected HTTP error {} when getting a token for {}",
                        other, self.body.auth.identity.password.user.name);
-                return Err(AuthError::HttpError(other, Some(resp_body)));
+                return Err(ApiError::HttpError(other, Some(resp_body)));
             }
         };
 
@@ -250,11 +199,16 @@ impl AuthMethod for IdentityAuthMethod {
         })
     }
 
-    /// Get a URL for the request service (NOT IMPLEMENTED).
-    fn get_endpoint(&mut self, _service_type: &str, _client: &Client)
-            -> Result<Url, AuthError> {
-        // TODO: implement
-        Err(AuthError::EndpointNotFound)
+    /// Get a URL for the requested service.
+    fn get_endpoint(&self, service_type: &str,
+                    endpoint_interface: Option<&str>,
+                    region: Option<&str>, client: &AuthenticatedClient)
+            -> Result<Url, ApiError> {
+        let real_interface = endpoint_interface.unwrap_or("public");
+        let cat = try!(catalog::get_service_catalog(&self.auth_url, client));
+        let endp = try!(catalog::find_endpoint(&cat, service_type,
+                                               real_interface, region));
+        endp.url.into_url().map_err(From::from)
     }
 }
 
