@@ -22,13 +22,22 @@
 
 #[macro_use]
 extern crate hyper;
+#[cfg(feature = "tls")]
 extern crate hyper_rustls;
 #[macro_use]
 extern crate log;
 #[macro_use]
 extern crate mime;
-extern crate rustc_serialize;
+extern crate serde;
+#[macro_use]
+extern crate serde_derive;
+extern crate serde_json;
 extern crate time;
 
 pub mod auth;
+mod common;
+pub mod identity;
+pub mod session;
 pub mod utils;
+
+pub use common::ApiError;
